# Author: Narayan Natarajan aka Venkman
# E-Mail: venkman1@ix.netcom.com
# $Date: 1998/02/08 23:55:13 $
# $Revision: 1.5 $ 
#
# USAGE : gawk -f expense.awk expense.txt
# Output: separate QIF files for each account that you can import to
#         MSMONEY (tested) or Quicken (not tested).
#
#
# 
# This is gawk script is freeware. Use it as you wish.
# I only have one request: Please do not delete my name off the top of
# this file.
# Please do add  yours if you make any contribution to this script.
#
# I would be glad to host your script along with this one if you do.
# 
# You can download gawk from :
# ftp://ftp.cdrom.com/pub/garbo/garbo_pc/unix/gawk2156.zip
#
# You can search for gawk at shareware.com 
#
# Search for: All Categories, gawk
#
#
# Standard disclaimers apply: I am not responsible for anything that this
# software may cause. USE AT YOUR OWN RISK!.
#
# If you wish to be very safe, backup your money-datafile and then use it.
#
#
BEGIN {FS="\t"; start=0 
discover=0
at_t=0
citivisa=0
checking=0
cash=0

# I am adding this to keep track of what came from the pilot
checknumber="Pilot"
IGNORECASE=1
# print empty string to nullify old information in files
printf( "") > "at_t.qif"
printf( "") > "discover.qif"
printf( "") > "citivisa.qif"
printf( "") > "cash.qif"
printf( "") > "checking.qif"
}
#
# Some basics for people who don't know awk
# the # marks a comment
#

# I search for the word Expenses, all records follow this
/Expenses/{

# something to say 'Expenses' has been found so process record
start=1 
# skip record that says expenses
next
}

# this is  an 'if' statement
( start == 1 ){

# each dollar-number combo is a field in the record

if ( $1 == "End" ) exit



# fields are as follows 
# field	Description
# 	1	Type of account: Personal, Business, etc.
#	2	Date
#	3	Payee: supplies, dinner, etc.
#	4	Account: visa, mastercard, etc.
#	5	Useless
#	6	Amount: multiply by -1 to get correct value
#	7	Vendor: somewhat like payee, but its what you write in the pilot
#	8	Junk
#	9	junk
#	10	Note, memo whatever.

# lets start mapping categories
# expense type personal business etc: pilot's categories
type=$1

# second is date
date=$2

# 3rd is Payee
payee=$3

# account type: visa, mastercard etc.
acctype=$4

# 5th, is useless, 
# 6 is amount : multiply by -1 to create a expense, otherwise its income
amount=$6 * -1

# 7 is vendor : add note field 10 and 7 together to create a note
vendor=$7

# 8,9 discard
# 10 note
memo=$10

# 7 is vendor : add note field 10 and 7 together to create a note
note=$7 " " $10

# if neither note field nor the vendor field exist: set note_exist to 0
if ( $10 == "" && $7 == "" ) note_exist = 0
else
	note_exist=1



# the squiggly character (tilde) is to search acctype for the word
# partial matches  allowed
# notice I have set IGNORECASE up there to be true (1)
# 
# these are my accounts
#  make your own mapping if you want with the following skeleton
#
#  MasterCard == AT&T MasterCard
#  Cash       == Cash
#  Visa       == Citibank Visa
#  Credit Card== Discover
#
#  Unfiled    == means that you did not pick any account
#                (maybe you could use that fact itself to define an account)
#
# Each account will have its own QIF file, thus transactions need
# to be separated
# this if block initiates the file with the words "!Type:<accountname>"
# this seems to tell MSMONEY Something which I don't understand. This
# seems to be pretty stupid because  you have to define the import account
# at MSMONEY's end any way.

if ( acctype == "" ) {
# this should not happen, but if it does tell the user
	print "Something weird going on: The account type is empty"
	next
}
if ( acctype ~ /MasterCard/ ) 
{
	account="at_t.qif"
	if ( at_t == 0 )
	{
		at_t=1
		print "!Type:CCard" > "at_t.qif"
		print "at&t created"
	}
}
if ( acctype ~ /Cash/ ) 
{
	account="cash.qif"
	if ( cash == 0 )
	{
		cash=1
		print "!Type:Cash" > "cash.qif"
		print "cash created"
	}
}
if ( acctype ~ /Visa/ ) 
{
	account="citivisa.qif"
	if ( citivisa == 0 )
	{
		citivisa=1
		print "!Type:CCard" > "citivisa.qif"
		print "visa created"
	}
}
if ( acctype ~ /Credit Card/ ) 
{
	account="discover.qif"
	if ( discover == 0 )
	{
		discover=1
		print "!Type:CCard" > "discover.qif"
		print "discover created"
	}
}
if ( acctype ~ /Check/ )  
{
	account="checking.qif"
	if ( checking == 0 )
	{
		checking=1
		print "!Type:Bank" > "checking.qif"
		print "checking created"
	}
}
if ( acctype ~ /Unfiled/ )
{
	print "WARNING: Skipping record because account is defined as Unfiled"
	print $0
	next
}



#  each of the following is one line
#  these are the QIF field prefixes:
#
#  D date
#  P Payee name
#  T Total? dunno but its the amount
#  M Memo probably
#  L List? well its the category for example-- Food:Groceries
#  N Number, as in check number: just a string
#  ^ Ends a record  
#        
# NOTE I am writing "Pilot" for number to easily keep track of records
#      imported from Pilot expense.
#
#   
#   
# Suggestion : Make your own mapping
# You  have personal, business etc and items such as dinner, supplies
# from this make your own category
# for example: if $1 (type) is personal and $7 (payee) is Dinner make
# the category:
#
# food:dining out
#
# or something like that and add it to construct below

# Clear any info that may have been stored from previous runs

Category=""
if ( type ~ /Personal/ )
{
	if ( payee ~ /breakfast/ ) Category = "Food:Dining Out"
	if ( payee ~ /dinner/ ) Category = "Food:Dining Out"
	if ( payee ~ /lunch/ ) Category = "Food:Dining Out"
	if ( payee ~ /snack/ ) Category = "Food:Dining Out"
	if ( payee ~ /entertainment/ ) {
		if ( vendor ~ /video/ || vendor ~ /movie/ )
			payee="Movie"
		Category = "Entertainment"
	}

	if ( payee ~ /gas/ ) Category = "Automobile:Gas"
	if ( payee ~ /postage/ ) Category = "Miscellaneous"
	if ( payee ~ /supplies/ ){

# Supplies is for Groceries
# What I am doing here is setting up major food stores
# I have frequented
		if ( vendor ~ /safeway/ ) payee="Safeway"
		if ( vendor ~ /q.f.c/ || vendor ~ /qfc/ ) payee="Q.F.C"
		if ( vendor ~ /meijer/ ) payee="Meijer"
		if ( vendor ~ /kroger/ ) payee="Kroger"
		if ( vendor ~ /Food town/ ) payee="Food town"
		if ( vendor ~ /L&L/ ) payee="L&L Food Center"
# no note field if its just groceries
		note_exist=0
		Category = "Food:Groceries"
	}
}

if ( type ~ /Business/ )
{
	if ( payee ~ /airfare/ ) Category = "Travel Expense:Miscellaneous"
	if ( payee ~ /breakfast/ ) Category = "Travel Expense:Food"
	if ( payee ~ /snack/ ) Category = "Travel Expense:Food"
	if ( payee ~ /dinner/ ) Category = "Travel Expense:Car Rental"
	if ( payee ~ /lunch/ ) Category = "Travel Expense:Car Rental"
	if ( payee ~ /car rental/ ) Category = "Travel Expense:Car Rental"
	if ( payee ~ /entertainment/ ) Category = "Travel Expense:Miscellaneous"
	if ( payee ~ /gas/ ) Category = "Travel Expense:Gas"
	if ( payee ~ /lodging/ ) Category = "Travel Expense:Hotel"
	if ( payee ~ /Hotel/ ) Category = "Travel Expense:Hotel"
	if ( payee ~ /parking/ ) Category = "Travel Expense:Parking"
	if ( payee ~ /telephone/ ) Category = "Travel Expense:Phone"
	if ( payee ~ /toll/ ) Category = "Travel Expense:Toll"
}

if ( type ~ /Unfiled/ )
{
	print "WARNING: Skipping record because Pilot Category is: Unfiled"
	print $0
}

# the \n marks a return character, so that each field is in one line
# each %s is a place holder for a variable such as date, payee, etc.
printf("D%s\nP%s\nT%s\nN%s\n", date, payee, amount, checknumber) >> account
if ( note_exist == 1 ) printf("M%s\n",note) >> account
if ( Category != "" ) printf("L%s\n", Category) >> account

# since the line has finished "hit the enter key"
printf("^\n") >> account


}
